//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
// P2Line.cpp: implementation of the CP2Line class.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Proj1.h"
#include "P2Line.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CP2Line::CP2Line()
//------------------------------------------------------------------------------
// Default Constructor.
{
	m_line.put(0.0,1.0,0.0);				// initially an x-axis line
											// (0x+ 1y + 0w = 0)
	m_lineColor = QS_RED;
	m_planeColor = QS_MAGENTA;
	m_drawStyle = 3;						// initially draw everything.
}

CP2Line::~CP2Line()
//------------------------------------------------------------------------------
// Default Destructor.
{
}

void CP2Line::drawMe(CquikColor *pLight)
//------------------------------------------------------------------------------
// Draw the line defined by E=(e1,e2,e3), either as the line in the x3=-1 plane
// or the plane through that line and the origin:
// m_drawStyle==0 draw nothing;  ==1 draw line; ==2 draw a plane; ==3 draw both.
//-------------
// HOW? Find where the line hits left, right, top, or bottom of a square box 
// given by (x1=+/-1, x2= +/-1, x3=-1), and draw the line there.  Remember
// that:		x1*e1 + x2*e2 + x3*e3 = 0, and be careful not to divide by zero:
// Top/bottom: x1*e1 + (+/-1)e2 + (-1)e3 = 0;
//				thus x1_bott  = (e3 + e2)/e1,   (x1 where line hits box bottom)
//					 x1_top   = (e3 - e2)/e1
// Left/right: (+/- 1)e1 + x2*e2 -e3 = 0;		)
//				thus x2_left  = (e3 + e1)/e2,	(x2 where line hits box left)
//				     x2_right = (e3 - e1)/e2
{
#define JT_RAYLENGTH 2.0
#define JT_EPSILON 10E-10					// smallest denominator allowed
double x1_bott,x1_top,x2_left,x2_right;		// intersections of line and box
double Ax,Ay,Bx,By;							// line-drawing coordinates.
int cnt;

	if(m_drawStyle==0) return;				// Drawing was disabled; do nothing.
										// Find where line intersects +/-1 box
	x1_bott = x1_top = x2_left = x2_right = -50.0;	// init to out-of-box vals.
	if(m_line.row[0]> JT_EPSILON || m_line.row[0] < -JT_EPSILON) 
	{									// if we can divide by e1,
		x1_bott  = (m_line.row[2] + m_line.row[1])/m_line.row[0];// (e3+e2)/e1,
		x1_top   = (m_line.row[2] - m_line.row[1])/m_line.row[0];// (e3-e2)/e1,
	}
	if(m_line.row[1]> JT_EPSILON || m_line.row[1] < -JT_EPSILON) 
	{									// if we can divide by  e2,
		x2_left  = (m_line.row[2] + m_line.row[0])/m_line.row[1];// (e3+e1)/e2,
		x2_right = (m_line.row[2] - m_line.row[0])/m_line.row[1];// (e3-e1)/e2.
	}
	// Choose endpoints A,B for line across the +/-1 box:
	cnt = 0;								// count # of endpoints found.
	if(x1_bott > -1.0 && x1_bott <=1.0)		// does line hit box bottom edge?
	{
		cnt = 1; Ax = x1_bott; Ay = -1.0;	// yes; we found endpoint A.
	}
	if(x1_top > -1.0 && x1_top <=1.0)		// does line hit box top edge?
	{
		switch(cnt)
		{
			case 0:							// yes; we found endpoint A.
				cnt++; Ax = x1_top; Ay = 1.0; break;
			case 1:							// yes; we found endpoint B.
				cnt++; Bx = x1_top; By = 1.0; break;
			default: break;
		}
	}
	if(x2_left > -1.0 && x2_left <=1.0)		// does line hit box left edge?
	{
		switch(cnt)
		{
			case 0:							// yes: we found endpoint A.
				cnt++; Ax = -1.0; Ay = x2_left; break;
			case 1:							// yes; we found endpoint B.
				cnt++; Bx = -1.0; By = x2_left; break;
			default: break;
		}
	}
	if(x2_right>-1.0 && x2_right <=1.0)		// does line hit box right edge?
	{
		switch(cnt)
		{
			case 0:							// yes; we found endpoint A
				cnt++; Ax = 1.0; Ay = x2_right; break;
			case 1:							// yes; we found endpoint B.
				cnt++; Bx = 1.0; By = x2_right; break;
			default: break;
		}
	}
	if(cnt==2)							// did we find 2 endpoints? draw!
	{
		if(m_drawStyle==1)
		{
			m_drawStyle=1;
		}
		if(m_drawStyle & 0x01)			// Draw a line on the x3=-1 plane
		{
			pLight->setLights(m_lineColor);	// set color.
			glBegin(GL_LINES);			// draw a line.:
				glVertex3d(Ax, Ay,-1.0);
				glVertex3d(Bx, By,-1.0);
				glVertex3d(0.0,0.0,0.0);
			glEnd();
		}
		if(m_drawStyle & 0x02)			// Draw a plane
		{
			pLight->setLights(m_planeColor);
			glBegin(GL_TRIANGLES);			// draw a triangle.
				glVertex3d(Ax*JT_RAYLENGTH, Ay*JT_RAYLENGTH,-1*JT_RAYLENGTH);
				glVertex3d(Bx*JT_RAYLENGTH, By*JT_RAYLENGTH,-1*JT_RAYLENGTH);
				glVertex3d(0.0,0.0,0.0);
			glEnd();
		}
	}
#undef JT_RAYLENGTH
}
