//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
// P2Point.cpp: implementation of the CP2Point class.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Proj1.h"
#include "P2Point.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CP2Point::CP2Point()
//------------------------------------------------------------------------------
// Default constructor.
{
	m_pt.put(0.0,0.0,-1.0);				// initially at center-of-projection
	m_ptColor = QS_YELLOW;			
	m_rayColor = QS_GREEN;
	m_drawStyle = 3;					// initially draw everything.
}

CP2Point::~CP2Point()
//------------------------------------------------------------------------------
// Default destructor.
{
}

void CP2Point::drawMe(CquikColor *pLight)
//------------------------------------------------------------------------------
// Draw the point, either as a small box, a ray from the origin thru the point,
// or both, chosen by m_drawStyle.
{
Vec3 tmp;
#define JT_RAYLENGTH 2.0
										// Decide what to draw:
	if(m_drawStyle & 0x01)				
	{									// Draw the point as a box.
		pLight->setLights(m_ptColor);	// Set the color,
		glMatrixMode(GL_MODELVIEW);		// Move origin to the point location,
		glPushMatrix();
		glTranslated(m_pt.row[0],m_pt.row[1],m_pt.row[2]);
		drawCube(0.008f);				// draw little cube there.
		glPopMatrix();					// restore the original matrix.
	}
	if(m_drawStyle & 0x02)				// Draw the point as a ray.
	{
		tmp.norm(m_pt);					// make a unit-length version of ray,
		tmp.scale(tmp,JT_RAYLENGTH);	// set length.(length from CquikGL.cpp)
		pLight->setLights(m_rayColor);	// Set the color.
	glBegin(GL_LINES);
		glVertex3d(tmp.row[0],tmp.row[1],tmp.row[2]);	// positive endpoint
//		glVertex3d(-tmp.row[0],-tmp.row[1],-tmp.row[2]);// negative endpoint. 
		// --OR--- //
		glVertex3d(0.0f,0.0f,0.0f);			// origin (uncomment only one)
	glEnd();
	}
}
#undef JT_RAYLENGTH

void CP2Point::drawCube(float sideLength)
//------------------------------------------------------------------------------
// Draw a cube at origin, sized by sideLength.  This is a home-made replacement
// for 'auxSolidCube()' function; the Windows implementation causes small 
// memory leaks and messes up lighting/materials settings.
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();					// Save current matrix
	glScalef(sideLength,sideLength,sideLength);	// scale to proper size
	glBegin(GL_QUADS);
	glVertex3f( 1.0f, 1.0f, 1.0f);	// +x
	glVertex3f( 1.0f,-1.0f, 1.0f);	
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);

	glVertex3f( 1.0f, 1.0f, 1.0f);	// +y
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);

	glVertex3f( 1.0f, 1.0f, 1.0f);	// +z
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -x
	glVertex3f(-1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f, 1.0f);
	glVertex3f(-1.0f, 1.0f,-1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -y
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f, 1.0f);
	glVertex3f(-1.0f,-1.0f, 1.0f);

	glVertex3f(-1.0f,-1.0f,-1.0f);	// -z
	glVertex3f(-1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f, 1.0f,-1.0f);
	glVertex3f( 1.0f,-1.0f,-1.0f);
	glEnd();
	glPopMatrix();					// Restore original matrix.
}