//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
// quikColor.cpp: collects together all lighting/materials calls for quikGL.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Proj1.h"
#include "quikColor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CquikColor::CquikColor()
//------------------------------------------------------------------------------
// Default constructor
{
	m_currentID = -1;							// Initially invalid color.
}

CquikColor::~CquikColor()
//------------------------------------------------------------------------------
// Default destructor
{
}

BOOL CquikColor::setLights(int colorID)
//------------------------------------------------------------------------------
// Change to color setup defined by 'colorID'. Invalid values return FALSE 
//	and do nothing.
{

	if(colorID==m_currentID) return(TRUE);	// Lights unchanged? do nothing.

	if(colorID < 1000)					
	{// Simple Color............................................................
		if(m_currentID >= 1000 || m_currentID<0)
		{								//Complex->Simple change................ 
			glShadeModel(GL_FLAT);			// Disable pixel-by-pixel shading
			glDisable(GL_LIGHTING);			// Turn off complex lighting,
			glDisable(GL_BLEND);			// Turn off surface transparency
		}
										//Simple->Simple change..................
		switch(colorID)
		{
			case QS_BLACK:
				glColor4ub(  0,  0,  0,255);	// black drawing color
				break;
			case QS_WHITE:
				glColor4ub(255,255,255,255);	// white drawing color
				break;
			case QS_RED:
				glColor4ub(255,  0,  0,255);	// red drawing color
				break;
			case QS_GREEN:
				glColor4ub(  0,255,  0,255);	// green drawing color
				break;
			case QS_BLUE:
				glColor4ub(  0,  0,255,255);	// blue drawing color
				break;
			case QS_CYAN:
				glColor4ub(  0,255,255,255);	// cyan drawing color
				break;
			case QS_MAGENTA:
				glColor4ub(255,  0,255,255);	// magenta drawing color
				break;
			case QS_YELLOW:
				glColor4ub(255,255,  0,255);	// yellow drawing color
				break;
			default:
				return FALSE;					// UNKNOWN LIGHT SETUP#!!!
				break;
		}
	}
	else// Complex lighting.....................................................
	{
// OpenGL transforms EVERYTHING by GL_MODELVIEW matrix, then GL_PROJECTION 
// matrix, then viewport matrix to stuff onscreen.  We want lighting to stay
// fixed to the camera's viewpoint, so we temporarily nullify the GL_MODELVIEW
// matrix by pushing its current value onto the matrix stack, replacing it with
// the identity matrix.  After all lighting settings are done, then pop the old
// MODELVIEW matrix off the stack before we draw anything.
		glMatrixMode(GL_MODELVIEW);				// Pick modelview matrix and
		glPushMatrix();							// push it onto its own stack.
		glLoadIdentity();						// Benign matrix--does nothing.
		error_GL("OnCreateGL::Light Coords");	// report any errors.
		// Complex->Complex lighting...........................................
		switch(colorID)
		{
		case QC_SILVER:
			light0();							// set our light,
			glDisable(GL_LIGHT1);				// disable other light 
			materialSilver();					// set our material.
			break;
		case QC_COPPER:
			light0();							// set our lights,
			light1();
			materialCopper();					// set our material.
			break;
		case QC_LEAFY:
			light1();							// set our light,
			glDisable(GL_LIGHT0);				// disable other light.
			materialLeafy();					// set our material.
			break;
		default:
			return FALSE;						// UNKNOWN LIGHT SETUP#!!!
			break;
		}
		glPopMatrix();							// restore MODELVIEW matrix.
		if(m_currentID<1000 || m_currentID<0)
		{// Simple->Complex lighting...........................................
			glEnable(GL_LIGHTING);				// Enable OpenGL lighting,
			glShadeModel(GL_SMOOTH);			// Enable pixel-by-pixel shading
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glEnable(GL_BLEND);					// Enable transparency.
		}
	}
	m_currentID = colorID;						// Save light setup# so we will
												// skip redundant calls.
//m_currentID = -1; //Diagnostics: Set color EVERY TIME WE'RE CALLED
	return TRUE;								// Done.
}

void CquikColor::light0()
//------------------------------------------------------------------------------	
// Set openGL's 'LIGHT0' light params, enable.
{
GLfloat lite0pos[4]= {0.5f, 0.5f, 0.5f, 1.0f};		// Light 0 position
GLfloat lite0amb[4]= {0.15f, 0.15f, 0.15f, 1.0f};	// Light 0 ambient illum.
GLfloat lite0dif[4]= {0.99f, 0.99f, 0.99f, 1.0f};	// Light 0 diffuse illum.
GLfloat lite0spec[4]={0.88f, 0.88f, 0.88f, 1.0f};	// Light 0 specular illum.

	//(up to 8 independent light sources allowed in OpenGL: GL_LIGHT0 - 7)
	glLightfv(GL_LIGHT0, GL_POSITION, lite0pos);	// set light 0 values;
	glLightfv(GL_LIGHT0, GL_AMBIENT, lite0amb);	
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lite0dif);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lite0spec);
	glEnable(GL_LIGHT0);						// turn on Light 0.
}

void CquikColor::light1()
//------------------------------------------------------------------------------	
// Set openGL's 'LIGHT1' light params, enable.
{
GLfloat lite1pos[4]= {-1.0f, 1.0f, 1.0f, 1.0f};		// Light 1 position
GLfloat lite1amb[4]= {0.15f, 0.15f, 0.15f, 1.0f};	// Light 1 ambient illum.
GLfloat lite1dif[4]= {0.66f, 0.66f, 0.66f, 1.0f};	// Light 1 diffuse illum.
GLfloat lite1spec[4]={0.88f, 0.88f, 0.88f, 1.0f};	// Light 1 specular illum.

	//(up to 8 independent light sources allowed in OpenGL: GL_LIGHT0 - 7)
	glLightfv(GL_LIGHT1, GL_POSITION, lite1pos);	// set light 1 values;
	glLightfv(GL_LIGHT1, GL_AMBIENT, lite1amb);	
	glLightfv(GL_LIGHT1, GL_DIFFUSE, lite1dif);
	glLightfv(GL_LIGHT1, GL_SPECULAR, lite1spec);
	glEnable(GL_LIGHT1);						// turn on Light 1.

}

void CquikColor::materialSilver()
//------------------------------------------------------------------------------	
{
	// Set up a surface material...............................................
/*	// Reddish
GLfloat mat_ambient[4] = {0.5f, 0.1f, 0.1f, 1.0f};	// set ambient glow
GLfloat mat_diffuse[4] = {1.0f, 0.2f, 0.2f, 1.0f};	// set diffuse reflectance
GLfloat mat_specular[4]= {1.0f, 1.0f, 1.0f, 1.0f};	// set specular reflectance
GLfloat mat_shininess[1] = {40.0f};					// set specular sharpness
*/
// Steely Gray
GLfloat mat_ambient[4] = {0.20f, 0.20f, 0.20f, 1.0f};	// set ambient glow
GLfloat mat_diffuse[4] = {0.40f, 0.40f, 0.40f, 1.0f};	// set diffuse reflectance
GLfloat mat_specular[4]= {0.99f, 0.99f, 0.99f, 1.0f};	// set specular reflectance
GLfloat mat_shininess[1] = {30.0f};					// set specular sharpness

											// for polygon FRONT surfaces only,
											// set amb, diff, spec reflectances.
	glMaterialfv(GL_FRONT, GL_AMBIENT,	mat_ambient); 
	glMaterialfv(GL_FRONT, GL_DIFFUSE,	mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS,mat_shininess);
											// for polygon BACK surfaces only,
											// set zero values for everything.
GLfloat nullFour[4] = {0.0f, 0.0f, 0.0f, 1.0f};		// 'black' values for
GLfloat nullOne[1] = {1.0f};						// back sides of polys.
	glMaterialfv(GL_BACK, GL_AMBIENT, nullFour);	// ?WHY DOESN'T THIS WORK?
	glMaterialfv(GL_BACK, GL_DIFFUSE, nullFour);	// check: is glLightModelf()
	glMaterialfv(GL_BACK, GL_SPECULAR, nullFour);	// GL_LIGHT_MODEL_TWO_SIDE?
	glMaterialfv(GL_BACK, GL_SHININESS,nullOne);

	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,1.0);	// enable different lighting
												// for backside of polys
}

void CquikColor::materialCopper()
//------------------------------------------------------------------------------	
{
GLfloat mat_ambient[4] = {0.5f, 0.1f, 0.1f, 1.0f};	// set ambient glow
GLfloat mat_diffuse[4] = {1.0f, 0.2f, 0.2f, 1.0f};	// set diffuse reflectance
GLfloat mat_specular[4]= {1.0f, 1.0f, 1.0f, 1.0f};	// set specular reflectance
GLfloat mat_shininess[1] = {20.0f};					// set specular sharpness

											// for polygon FRONT surfaces only,
											// set amb, diff, spec reflectances.
	glMaterialfv(GL_FRONT, GL_AMBIENT,	mat_ambient); 
	glMaterialfv(GL_FRONT, GL_DIFFUSE,	mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS,mat_shininess);
											// for polygon BACK surfaces only,
											// set zero values for everything.
GLfloat nullFour[4] = {0.0f, 0.0f, 0.0f, 1.0f};		// 'black' values for
GLfloat nullOne[1] = {1.0f};						// back sides of polys.
	glMaterialfv(GL_BACK, GL_AMBIENT, nullFour);	// ?WHY DOESN'T THIS WORK?
	glMaterialfv(GL_BACK, GL_DIFFUSE, nullFour);	// check: is glLightModelf()
	glMaterialfv(GL_BACK, GL_SPECULAR, nullFour);	// GL_LIGHT_MODEL_TWO_SIDE?
	glMaterialfv(GL_BACK, GL_SHININESS,nullOne);

	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,1.0);	// enable different lighting
												// for backside of polys
}


void CquikColor::materialLeafy()
//------------------------------------------------------------------------------	
{
GLfloat mat_ambient[4] = {0.1f, 0.4f, 0.1f, 1.0f};	// set ambient glow
GLfloat mat_diffuse[4] = {0.2f, 1.0f, 0.2f, 1.0f};	// set diffuse reflectance
GLfloat mat_specular[4]= {0.8f, 1.0f, 0.8f, 1.0f};	// set specular reflectance
GLfloat mat_shininess[1] = {60.0f};					// set specular sharpness

											// for polygon FRONT surfaces only,
											// set amb, diff, spec reflectances.
	glMaterialfv(GL_FRONT, GL_AMBIENT,	mat_ambient); 
	glMaterialfv(GL_FRONT, GL_DIFFUSE,	mat_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS,mat_shininess);
											// for polygon BACK surfaces only,
											// set zero values for everything.
GLfloat nullFour[4] = {0.0f, 0.0f, 0.0f, 1.0f};		// 'black' values for
GLfloat nullOne[1] = {1.0f};						// back sides of polys.
	glMaterialfv(GL_BACK, GL_AMBIENT, nullFour);	// ?WHY DOESN'T THIS WORK?
	glMaterialfv(GL_BACK, GL_DIFFUSE, nullFour);	// check: is glLightModelf()
	glMaterialfv(GL_BACK, GL_SPECULAR, nullFour);	// GL_LIGHT_MODEL_TWO_SIDE?
	glMaterialfv(GL_BACK, GL_SHININESS,nullOne);

	glLightModelf(GL_LIGHT_MODEL_TWO_SIDE,1.0);	// enable different lighting
												// for backside of polys
}

void CquikColor::error_GL(const char *label)
//------------------------------------------------------------------------------
// If OpenGL had any errors, print string found at 'label' to report, and then 
// print the error report. Otherwise do nothing.
// (NOTE: the 'TRACE()' macro works just like the 'printf()' function, EXCEPT 
//    output appears at bottom of the VisualC++6.0 window on the 'debug' tab.)
{
GLenum errorno;								// OpenGL error code.

	errorno = glGetError();					
	if (errorno != GL_NO_ERROR)				// was there any unreported error?
	{										// YES: tell me about it.
		TRACE("%s had error: #(%d) %s\r\n", 
		label, errorno, gluErrorString(errorno)); // (give OpenGL's explanation)
	}
}