//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
//	quikColor.h
//
//	  A CquikColor object collects together all the color-setting calls used in
// CquikGL, including those for lighting and materials settings, to help avoid 
// bugs and redundant calls from misunderstood in openGL lighting states. The 
// one-and-only CquikColor object (CquikGL.m_color) defines a small number of 
// 'color setups' that determine on-screen colors as subsequent openGL draws 
// objects on-screen.  Users simply choose a desired color setup#, and 
// CquikColor makes the necessary openGL color,lighting, and materials calls.
//    Two kinds of color setups are used: 'simple' setups just choose an 
// on-screen drawing color directly (be sure to use them INSTEAD of glColor() 
// calls in your code, so that glColor can keep track of them for you).  The 
// 'Complex' determine on-screen color from light positions, and ambient, 
// diffuse, and specular terms for each light source and surface. 
//    The CquikColor class defines just a few fixed lighting setups at compile
// time; if you want to change lighting and materials properties at runtime
// then you will need to modify or abandon this class, but it should be
// sufficient for many uses.
//
//	04/23/2003 Jack Tumblin: created.
//

#if !defined(AFX_QUIKCOLOR_H__98137078_E572_4779_8B63_CA81CB23C527__INCLUDED_)
#define AFX_QUIKCOLOR_H__98137078_E572_4779_8B63_CA81CB23C527__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Define ID#s for lighting setups; Add more if you like, but be sure that:
//	Simple lighting setup #s must be >=0, <1000.
//  Complex lighting setup #s must be sequentially numbered, >=1000, and 
//		QC_MAX defines the array size needed to store them all.
// Simple lighting setups:
#define QS_BLACK	0		// RGBA =   0,  0,  0,  1
#define QS_WHITE	1		// RGBA =   1,  1,  1,  1
#define QS_RED		2		// RGBA =   1,  0,  0,  1
#define QS_GREEN	3		// RGBA =   0,  1,  0,  1
#define QS_BLUE		4		// RGBA =   0,  0,  1,  1
#define QS_CYAN		5		// RGBA =   0,  1,  1,  1
#define QS_MAGENTA	6		// RGBA =   1,  0,  1,  1
#define QS_YELLOW	7		// RGBA =   1,  1,  0,  1
#define QS_MAX		8		// total# of simple lighting setups.
// Complex lighting setups:
#define QC_SILVER	1000	// Side-lit, shiny, semi-transparent objects. 
#define QC_COPPER	1001	// Side-lit, shiny reddish opaque objects.
#define QC_LEAFY	1002	// Side-lit, shiny bright green objects.
#define QC_MAX		   3	// total # of complex lighting setups.

class CquikColor  
//==============================================================================
// Class to encapsulate a small # of openGL lighting states in an easy-to-change
// form; just give the lighting setup# you want.
//
{
public:
	CquikColor();							// Default constructor.
	virtual ~CquikColor();					// Default destructor.

	BOOL setLights(int lightID);			// Change to lighting setup given
											// by 'lightID'. Invalid values
											// Returns FALSE and do nothing.
private:
	int m_currentID;						// Currently applied lighting ID#.

	void error_GL(const char *label);		// Report any openGL errors.
	void light0(void);						// Key light position settings
	void light1(void);						// Fill light position settings
	void materialSilver(void);				// a materials-setting fcn.
	void materialCopper(void);				// a materials-setting fcn.
	void materialLeafy(void);				// a materials-setting fcn.

};

#endif // !defined(AFX_QUIKCOLOR_H__98137078_E572_4779_8B63_CA81CB23C527__INCLUDED_)
