//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
// NOTE!!! relies on quikGL class!!!
//
// quikMeshImg.h: 'rubber sheet' class that holds an image texture mapped onto
// a grid of quadrilaterals.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QUIKMESHIMG_H__FE0F53A2_3C3D_4B3D_8B0A_FA42D7EC4B60__INCLUDED_)
#define AFX_QUIKMESHIMG_H__FE0F53A2_3C3D_4B3D_8B0A_FA42D7EC4B60__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CquikBMP
//==============================================================================
// Class to read and write an image in Windows' strange 'BMP' file format.
// The 'm_pImg' member points to a single RGB image buffer, used for both
// reading and writing BMP images. 
// Currently only reads/writes 24-bit RGB images without compression.
{
private:
	BITMAPFILEHEADER m_BMPone;				// 1st & (ptr to) 2nd structure at
	BITMAPINFO		 *m_pBMPtwo;			// the start of every BMP file.
	int m_xsize,m_ysize;					// image size given in pixels.

	BYTE *m_pImg;							// Dynamically alloc'd image buffer.
											// (Set to NULL if no memory taken)
											// holds BYTE-sized, RGB triplets
											// BUT (like BMP format) scanlines
											// are multiple-of-4 bytes each.
	int m_linesize;							// Size of a scanline in bytes;
											
public:										// All fcns return FALSE on error.
	CquikBMP();								// Default constructor/destructor
	~CquikBMP();
	void wipe();							// discard any current contents.

	int getXsize(void){return m_xsize;};		// Read image size;
	int getYsize(void){return m_ysize;};
	int getLinesize(void){return m_linesize;};
	BOOL sizer(int Xsize, int Ysize);		// Re-size image buffer to prepare
											// to write into it. Any size change
											// will discard buffer's contents.
	BOOL fileRead(OPENFILENAME *pName);		// Open, read, & close named file.
											// (Discards any existing contents)
	BOOL fileWrite(OPENFILENAME *pName);	// Open, write, & close named file
											// using current buffer contents.

	BYTE getPixel(int x, int y, int z);		// Read image buffer; z=0 for blue,
											// z=1 for green, z=2 for red.
	BYTE putPixel(int x, int y, int z, BYTE val);// Set an image buffer value
	BYTE* getPtr(int x, int y);				// Return pointer to image buffer
											// at location x,y; pixels stored in
											// BGR (not RGB) order (!).
};

class CquikPLY
//==============================================================================
// Class to read and write a 3D polygonal mesh in Greg Turk's 'PLY' format.
// The m_pV, m_pN, m_pT members point to buffers for vertices, normals, and
// texture coords respectively, used for both reading and writing PLY files.
{
};

class CquikMeshImg  
//==============================================================================
// 3D distortable image; holds a rectangular grid of vertices and a texture map
// image that is applied to it.  Initially a flat 'billboard' placed in 3D, 
// changing vertex positions in grid stretches to any 3D shape.  
{
public:	//-------------Assignment A,B:
	CP2Line	 m_lines[4];					// User positioned lines:
											// (keys 5,6,7,8)
	CP2Point m_inPts[4];					// User-positioned point pairs: 
	CP2Point m_outPts[4];					// input points (x) are transformed
											// by m_H before display; output
											// points (x') are not.

	int m_pick;								// Code for selected line or point;
											// 0== none; = 1,2,3,4 for points,
											// ==-1,-2,-3,-4 for lines.
	CP2H	m_H;							// used for meshImg object display.
	CP2H	m_Hnew;							// copied to m_H when 'H' key down.

public:
	CquikMeshImg();							// Constructor, initializer
	virtual ~CquikMeshImg();				// Destructor (default)

											// Texture Image:..................
	CquikBMP m_BMP;							// BMP-format image we'll use.
	GLint m_Txsize,m_Tysize,m_Tzsize;		// # of pixels, # of colors/pix
											// (MUST be 2^n x 2^n)
	GLubyte *m_pTimg;						// Ptr to the texture image
											// Mesh:..........................
	GLint m_Mxsize,m_Mysize;				// # of vertices in the mesh.
		
	CquikColor *m_pLight;					// Ptr to GL lighting/color info.

private:
	GLfloat *m_pNorm, *m_pTexy, *m_pVert;	// pointers to dyn. alloc'd array 
											// of normal vectors, texture coords
											// and vertices for the mesh.
	float m_yTexmax, m_xTexmax;				// Largest texture-map coords that
											// are within the BMP image.
	BOOL m_isNewTexture;					// ==TRUE; m_pTimg holds new texture
											// image that openGL needs to load.
											// (drawMe() loads it, clears flag)

public:
	BOOL fileSave(OPENFILENAME *pName);		// Save current texture map to file
											// <*pName>.bmp, and mesh geometry 
											// to <*pName>.ply
	BOOL fileLoad(OPENFILENAME *pName);		// Load texture map from file 
											// <*pName>.bmp and save mesh geom. 
											// from <*pName>.ply
	BOOL sizerTex(int Ximg,int Yimg,int Zimg);// Set texture map size: CAUTION
											// power-of-two sizes only! Any size
											// change discards current contents.
	BOOL sizerMesh(int Xmesh, int Ymesh);	// Set # of vertices in mesh. Size
											//changes discard current positions.

	void wipeTex(void);						// Discard any contents, release
											// all texture memory allocated.
	void wipeMesh(void);					// Discard any contents, release
											// all mesh memory allocated.

	void makeTestImgMeshDisplayList(void);	// Generate a test pattern texture.				
	void makeTestImgMesh(void);				// Generate a test pattern texture.
	void drawMe(void);						// call this only from within 
											// CquikGL::drawOpenGL() only.
	void meshWrinkle(float zoffset);		// Offset alternating mesh verts.			

private:
	void setupTexParams(void);				// Setup openGL texture map state 
											// to fit new/changed texture image.
	void error_GL(char *label);				// Report last openGL error, if any
											//(same as quikGL::errorGL())
};

#endif // !defined(AFX_QUIKMESHIMG_H__FE0F53A2_3C3D_4B3D_8B0A_FA42D7EC4B60__INCLUDED_)
